#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "dog_dimmer.h"
#include "bitmap.h"


//from wsskiller
__inline void YUV2RGB(word yy, word uu, word vv, byte *r, byte *g, byte *b)
{
	signed int _r,_g,_b;
	signed int y, u, v;

//	y = yy;
	y = yy - 12;            //why 12?, wikipedia says 16
//	y = yy - 16;
	u = (int)uu - 128;
	v = (int)vv - 128;

	_r = _R(y,u,v);
	_g = _G(y,u,v);
	_b = _B(y,u,v);

	*r = _S(_r);
	*g = _S(_g);
	*b = _S(_b);
}

//convert from 1555 to 24 bit
int convert_bitmap_1555_24(word *bitmap1555, int width, int height, byte *bitmap24)
{
    int x,y,yy;

    //Rearrange the frame buffer to be BMP compatible
    for (y=0; y<height; y++) {
        yy = height - 1 - y;
        for (x=0; x<width; x++) {
            bitmap24[3*width*y + 3*x + 0] = (bitmap1555[width*yy + x]>>7)  & 0xf8;      //r
            bitmap24[3*width*y + 3*x + 1] = (bitmap1555[width*yy + x]>>2)  & 0xf8;      //g
            bitmap24[3*width*y + 3*x + 2] = (bitmap1555[width*yy + x]<<3)  & 0xf8;      //b
        }
    }
    return 0;
}

//convert from 1555 to 16 bit
int convert_bitmap_1555_16(word *bitmap1555, int width, int height, byte *bitmap16)
{
    int x,y,yy;

    //Rearrange the frame buffer to be BMP compatible
    for (y=0; y<height; y++) {
        yy = height - 1 - y;
        for (x=0; x<width; x++) {
            bitmap16[2*width*y + 2*x + 0] = (bitmap1555[width*yy + x]>>0) & 0xff;
            bitmap16[2*width*y + 2*x + 1] = (bitmap1555[width*yy + x]>>0) & 0xff;
        }
    }
    return 0;
}

//convert from 8 bit to 1555
int convert_bitmap_8_1555(byte *bitmap8, byte *palette, int width, int height, word *bitmap1555)
{
    int x,y,yy;
    int r,g,b;

    //Rearrange the frame buffer to be BMP compatible
    for (y=0; y<height; y++) {
        yy = height - 1 - y;
        for (x=0; x<width; x++) {
            r = (palette[4*bitmap8[width*yy + x]+0]>>3)&0x1f;
            g = (palette[4*bitmap8[width*yy + x]+1]>>3)&0x1f;
            b = (palette[4*bitmap8[width*yy + x]+2]>>3)&0x1f;

            bitmap1555[width*y + x] = (0x1<<15) | (r<<10) | (g<<5) | (b<<0);
        }
    }
    return 0;
}


//save 8/16/24/32 bit bitmap
int save_bitmap(int bytes_per_pixel, byte *bitmap, int width, int height)
{
    int channel_type;
    int channel_number;
	TYPE_TapChInfo channel_info;
    TYPE_File *fpY;
    char FileNameY[30];
    dword x, y;
    int i;
    long size, offset;
    struct BMP_Head Head;
    struct BMP_Info Info;
    byte Palette[1024];
    int palette_valid;

    palette_valid = (bytes_per_pixel == 1) ? 1 : 0;

    //Look for an unused file name
    TAP_Channel_GetCurrent(&channel_type, &channel_number);
    TAP_Channel_GetInfo(channel_type, channel_number, &channel_info);
    for (i=0; i<100; i++) {
      TAP_SPrint(FileNameY, "%s_dog_%d_%2.2d.bmp", channel_info.chName, 8*bytes_per_pixel, i);
      if (!TAP_Hdd_Exist(FileNameY)) break;
    }
    if (i == 100) {
        TAP_Print("\ntoo many files exist: %s", FileNameY);
        return -1;
    }
    TAP_Hdd_Create(FileNameY, ATTR_NORMAL);
    fpY = TAP_Hdd_Fopen(FileNameY);
    if (fpY == 0) {
        TAP_Print("\ncannot open: %s", FileNameY);
        return -1;
    }

    //BMP Header creation taken from the Topfield ScreenCapture sample
    offset = sizeof(Head) + sizeof(Info) + ((palette_valid) ? sizeof(Palette) : 0);
    size = offset + (bytes_per_pixel*width * height);
    memset(&Head, 0, sizeof(Head));
    Head.id[0]          = 'B';
    Head.id[1]          = 'M';
    Head.size           = LE32(size);
    Head.offset         = LE32(offset);

    memset(&Info, 0, sizeof(Info));
    Info.info_size      = LE32(0x28);
    Info.width          = LE32(width);
    Info.height         = LE32(height);
    Info.plane          = LE16(1);
    Info.bits_per_pixel = LE16(bytes_per_pixel * 8);
    Info.compress       = LE32(0);
    Info.img_size       = LE32(width * height);
    Info.x_res          = 0;
    Info.y_res          = 0;
    Info.color          = 0;
    Info.icolor         = 0;

    if (palette_valid) {
        //Initialise the grey scale palette
        memset(Palette, 0, sizeof(Palette));
        for (i=0; i<256; i++) {
          Palette[4*i + 0] = i;
          Palette[4*i + 1] = i;
          Palette[4*i + 2] = i;
        }
    }

    TAP_Hdd_Fwrite(&Head, sizeof(Head), 1, fpY);
    TAP_Hdd_Fwrite(&Info, sizeof(Info), 1, fpY);
    if (palette_valid) TAP_Hdd_Fwrite(Palette, sizeof(Palette), 1, fpY);
    TAP_Hdd_Fwrite(bitmap, bytes_per_pixel*width * height, 1, fpY);
    TAP_Hdd_Fclose(fpY);

    return 0;
}


//
//8 bit greyscale only ...
//
//unfinished...
int load_bitmap()
{
    TYPE_File *fpY;
    char FileNameY[30];
    int width;
    int height;
    int bytes_per_pixel;
    byte *bitmap;
    int i;
    int channel_type;
    int channel_number;
	TYPE_TapChInfo channel_info;
    int setpos_x, setpos_y, setpos_width, setpos_height;
    struct BMP_Head Head;
    struct BMP_Info Info;
    byte Palette[1024];
    int palette_valid;
    int ret;
    word value;
    int x,y,xx,yy;

    TAP_Channel_GetCurrent(&channel_type, &channel_number);
    TAP_Channel_GetInfo(channel_type, channel_number, &channel_info);
    TAP_SPrint(FileNameY, "%s_dog_%d_%2.2d.bmp", channel_info.chName, 8, 0);
    TAP_Print("\nloading %s", FileNameY);
    for (i=0; i<100; i++) {
        TAP_SPrint(FileNameY, "%s_dog_%d_%2.2d.bmp", channel_info.chName, 8, i);
        if (TAP_Hdd_Exist(FileNameY)) break;
    }
    if (i == 100) {
        TAP_Print("\ncan't find %s", FileNameY);
        return -1;
    }
    fpY = TAP_Hdd_Fopen(FileNameY);
    if (fpY == 0) {
        TAP_Print("\ncannot open: %s", FileNameY);
        return -1;
    }

    TAP_Hdd_Fread(&Head, sizeof(Head), 1, fpY);
    TAP_Hdd_Fread(&Info, sizeof(Info), 1, fpY);
TAP_Print("\nsize=%x", LE32(Head.size));
TAP_Print("\noffset=%x", LE32(Head.offset));

    width = LE32(Info.width);
    height = LE32(Info.height);
    bytes_per_pixel = LE16(Info.bits_per_pixel)/8;
    palette_valid = (bytes_per_pixel == 1) ? 1 : 0;

    if (palette_valid) TAP_Hdd_Fread(Palette, sizeof(Palette), 1, fpY);
    bitmap = TAP_MemAlloc(bytes_per_pixel * width * height);
    TAP_Hdd_Fseek(fpY, 0, LE32(Head.offset));
    ret = TAP_Hdd_Fread(bitmap, bytes_per_pixel*width * height, 1, fpY);
    TAP_Hdd_Fclose(fpY);

    if (palette_valid) {
    }

TAP_Print("\nbytes_per_pixel=%d, width=%d, height=%d", bytes_per_pixel, width, height);


    //display loaded dog

    //draw dog
    setpos_x      = (channel->getpos.x      * 720) / fw_aspect[CHANNEL_Main].horizontal_size;
    setpos_y      = (channel->getpos.y      * 576) / fw_aspect[CHANNEL_Main].vertical_size;
    setpos_width  = (channel->getpos.width  * 720) / fw_aspect[CHANNEL_Main].horizontal_size;
    setpos_height = (channel->getpos.height * 576) / fw_aspect[CHANNEL_Main].vertical_size;


    //convert from bitmap format
    //invert the dog
    //make it transparent
    for (y=0; y<setpos_height; y++) {
        yy = setpos_height - 1 - y;
        for (x=0; x<setpos_width; x++) {
            xx = (x * fw_aspect[CHANNEL_Main].horizontal_size) / 720;       //get position

            //transform: black->transparent, white->black  (aasumes nice greyscale palette)
            bitmap_mask_1555[setpos_width*y+x] = (bitmap[width*yy + xx] > 0x80) ? 0x8000 : 0x0000;
        }
    }


    //make sure the osd window is initialised
	if (rgn == 0) rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);

    TAP_Osd_DrawPixmap(rgn, setpos_x, setpos_y, setpos_width, setpos_height, bitmap_mask_1555, TRUE, OSD_1555);


    TAP_MemFree(bitmap);
    return 0;
}

